@@ -15,17 +15,23 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with Checkbox.  If not, see <http://www.gnu.org/licenses/>.
-#
+
+from __future__ import absolute_import
+from __future__ import division
+from __future__ import print_function
+from __future__ import unicode_literals
+
 from io import StringIO
+from io import open
+from unittest import TestCase
 
 from pkg_resources import resource_filename
-from unittest import TestCase
 
 from checkbox_support.parsers.udevadm import UdevadmParser, decode_id
 from checkbox_support.parsers.udevadm import parse_udevadm_output
 
 
-class DeviceResult:
+class DeviceResult(object):
 
     def __init__(self):
         self.devices = []
@@ -41,7 +47,7 @@ class DeviceResult:
         return None
 
 
-class UdevadmDataMixIn:
+class UdevadmDataMixIn(object):
     """
     Mix in with a helper method to load sample udevadm data
     """
@@ -239,12 +245,12 @@ E: UDEV_LOG=3
 
     def test_DELL_VOSTROV131(self):
         devices = self.parse("DELL_VOSTROV131")
-        expected_devices = [("RTL8111/8168 PCI Express Gigabit "
-                            "Ethernet controller",
-                            "NETWORK", "pci", 0x10EC, 0x8168),
-                            ("AR9285 Wireless Network Adapter (PCI-Express)",
-                            "WIRELESS", "pci", 0x168C, 0x002B)
-                            ]
+        expected_devices = [
+            ("RTL8111/8168 PCI Express Gigabit Ethernet controller",
+             "NETWORK", "pci", 0x10EC, 0x8168),
+            ("AR9285 Wireless Network Adapter (PCI-Express)",
+             "WIRELESS", "pci", 0x168C, 0x002B)
+        ]
         self.assertEqual(len(devices), 63)
         self.assertEqual(self.count(devices, "VIDEO"), 1)
         self.assertEqual(self.count(devices, "AUDIO"), 2)
@@ -284,6 +290,48 @@ E: UDEV_LOG=3
         self.assertEqual(self.count(devices, "NETWORK"), 1)
         self.assertEqual(self.count(devices, "WIRELESS"), 1)
 
+    def test_DELL_INSPIRON_7737_NVIDIA(self):
+        devices = self.parse("DELL_INSPIRON_7737_NVIDIA")
+        expected_devices = [(None,
+                             "WIRELESS", "pci", 0x8086, 0x08b1),
+                            (None,
+                             "VIDEO", "pci", 0x10de, 0x0fe4),
+                            (None,
+                             "VIDEO", "pci", 0x8086, 0x0a16)
+                            ]
+        # The first video device is an NVIDIA GPU, which is too new
+        # to have a  device name. The second one is the built-in Haswell
+        # GPU.
+        self.assertEqual(len(devices), 59)
+        self.assertEqual(self.count(devices, "WIRELESS"), 1)
+        self.assertEqual(self.count(devices, "BLUETOOTH"), 1)
+        self.assertEqual(self.count(devices, "NETWORK"), 1)
+        self.assertEqual(self.count(devices, "VIDEO"), 2)
+        self.assertEqual(self.count(devices, "AUDIO"), 4)
+        self.assertEqual(self.count(devices, "DISK"), 1)
+        self.verify_devices(devices, expected_devices)
+
+    def test_DELL_INSPIRON_3048_AMD(self):
+        devices = self.parse("DELL_INSPIRON_3048")
+        expected_devices = [(None,
+                             "WIRELESS", "pci", 0x168c, 0x0036),
+                            (None,
+                             "VIDEO", "pci", 0x1002, 0x6664),
+                            (None,
+                             "VIDEO", "pci", 0x8086, 0x0402)
+                            ]
+        # The first video device is an AMD GPU, which is too new
+        # to have a  device name. The second one is the built-in Haswell
+        # GPU.
+        self.assertEqual(len(devices), 63)
+        self.assertEqual(self.count(devices, "WIRELESS"), 1)
+        self.assertEqual(self.count(devices, "BLUETOOTH"), 1)
+        self.assertEqual(self.count(devices, "NETWORK"), 1)
+        self.assertEqual(self.count(devices, "AUDIO"), 4)
+        self.assertEqual(self.count(devices, "DISK"), 1)
+        self.assertEqual(self.count(devices, "VIDEO"), 2)
+        self.verify_devices(devices, expected_devices)
+
     def test_HOME_MADE(self):
         devices = self.parse("HOME_MADE")
         self.assertEqual(len(devices), 72)
@@ -352,6 +400,8 @@ E: UDEV_LOG=3
         self.assertEqual(self.count(devices, "RAID"), 0)
         self.assertEqual(self.count(devices, "DISK"), 3)
         self.assertEqual(self.count(devices, "NETWORK"), 1)
+        expected_devices = [(None, "VIDEO", "pci", 0x8086, 0x2E32)]
+        self.verify_devices(devices, expected_devices)
 
     def test_HP_PROBOOK6550B_ACCELEROMETER(self):
         devices = self.parse("HP_PROBOOK6550B_ACCELEROMETER")
@@ -405,7 +455,7 @@ E: UDEV_LOG=3
     def test_LENOVO_E445(self):
         devices = self.parse("LENOVO_E445")
         self.assertEqual(len(devices), 78)
-        self.assertEqual(self.count(devices, "VIDEO"), 1)
+        self.assertEqual(self.count(devices, "VIDEO"), 2)
         self.assertEqual(self.count(devices, "AUDIO"), 4)
         self.assertEqual(self.count(devices, "KEYBOARD"), 1)
         self.assertEqual(self.count(devices, "TOUCHPAD"), 1)
@@ -421,6 +471,11 @@ E: UDEV_LOG=3
         self.assertEqual(self.count(devices, "CAPTURE"), 1)
         self.assertEqual(self.count(devices, "NETWORK"), 1)
         self.assertEqual(self.count(devices, "WIRELESS"), 1)
+        # System has two CPUs, AMD Richland [Radeon HD 8650G] and
+        # Sun PRO [Radeon HD 8570A/8570M]
+        expected_devices = [(None, "VIDEO", "pci", 0x1002, 0x990b),
+                            (None, "VIDEO", "pci", 0x1002, 0x6663)]
+        self.verify_devices(devices, expected_devices)
 
     def test_LENOVO_T430S(self):
         devices = self.parse("LENOVO_T430S")
@@ -571,6 +626,89 @@ E: UDEV_LOG=3
         self.assertEqual(self.count(devices, "RAID"), 0)
         self.assertEqual(self.count(devices, "DISK"), 1)
 
+    def test_IBM_PSERIES_P8(self):
+        # Apparnently a virtualized system on a pSeries P8
+        # Quite bare-bones, server-oriented system
+        devices = self.parse("IBM_PSERIES_POWER7")
+        self.assertEqual(self.count(devices, "VIDEO"), 0)
+        self.assertEqual(self.count(devices, "AUDIO"), 0)
+        self.assertEqual(self.count(devices, "KEYBOARD"), 0)
+        self.assertEqual(self.count(devices, "TOUCHPAD"), 0)
+        self.assertEqual(self.count(devices, "CARDREADER"), 0)
+        self.assertEqual(self.count(devices, "CDROM"), 1)
+        self.assertEqual(self.count(devices, "FIREWIRE"), 0)
+        self.assertEqual(self.count(devices, "MOUSE"), 0)
+        self.assertEqual(self.count(devices, "ACCELEROMETER"), 0)
+        self.assertEqual(self.count(devices, "TOUCHSCREEN"), 0)
+        self.assertEqual(self.count(devices, "WIRELESS"), 0)
+        self.assertEqual(self.count(devices, "NETWORK"), 1)
+        self.assertEqual(self.count(devices, "BLUETOOTH"), 0)
+        self.assertEqual(self.count(devices, "CAPTURE"), 0)
+        self.assertEqual(self.count(devices, "RAID"), 0)
+        self.assertEqual(self.count(devices, "DISK"), 2)
+        self.assertEqual(len(devices), 4)
+
+    def test_QEMU_KVM(self):
+        # A virtual machine, QEMU-KVM-based. Some of its devices are those
+        # of the host system, we're interested mainly in network and disk
+        # devices (http://pad.lv/1355282)
+        devices = self.parse("QEMU_KVM")
+        self.assertEqual(len(devices), 23)
+        self.assertEqual(self.count(devices, "VIDEO"), 1)
+        self.assertEqual(self.count(devices, "AUDIO"), 0)
+        self.assertEqual(self.count(devices, "KEYBOARD"), 1)
+        self.assertEqual(self.count(devices, "CARDREADER"), 0)
+        self.assertEqual(self.count(devices, "CDROM"), 0)
+        self.assertEqual(self.count(devices, "FIREWIRE"), 0)
+        self.assertEqual(self.count(devices, "MOUSE"), 1)
+        self.assertEqual(self.count(devices, "WIRELESS"), 0)
+        self.assertEqual(self.count(devices, "NETWORK"), 1)
+        self.assertEqual(self.count(devices, "BLUETOOTH"), 0)
+        self.assertEqual(self.count(devices, "CAPTURE"), 0)
+        self.assertEqual(self.count(devices, "RAID"), 0)
+        self.assertEqual(self.count(devices, "DISK"), 1)
+        self.assertEqual(self.count(devices, "SCSI"), 1)
+
+
+
+    def test_DELL_VOSTRO_270(self):
+        # Interesting because while its Intel video card has the same PCI
+        # vendor/product ID as others (8086:0152) the subvendor_id and
+        # subproduct_id attributes were causing it to not be recognized as
+        # video.  HOWEVER, we can't just assume that all Intel video cards are
+        # doing the same, so some creative quirking will be needed in the
+        # parser to single these out.  It's a desktop system so no touchpad and
+        # has an external mouse.  The card reader is not detected as such,
+        # instead it appears as about 11 disk devices.
+        # Finally, it's a hybrid video system with a second Nvidia GPU.
+        devices = self.parse("DELL_VOSTRO_270")
+        self.assertEqual(self.count(devices, "VIDEO"), 2)
+        self.assertEqual(self.count(devices, "AUDIO"), 4)
+        self.assertEqual(self.count(devices, "KEYBOARD"), 1)
+        self.assertEqual(self.count(devices, "TOUCHPAD"), 0)
+        self.assertEqual(self.count(devices, "CARDREADER"), 0)
+        self.assertEqual(self.count(devices, "CDROM"), 1)
+        self.assertEqual(self.count(devices, "FIREWIRE"), 0)
+        self.assertEqual(self.count(devices, "MOUSE"), 1)
+        self.assertEqual(self.count(devices, "ACCELEROMETER"), 0)
+        self.assertEqual(self.count(devices, "TOUCHSCREEN"), 0)
+        self.assertEqual(self.count(devices, "DISK"), 12)
+        self.assertEqual(self.count(devices, "CAPTURE"), 0)
+        self.assertEqual(self.count(devices, "RAID"), 0)
+        self.assertEqual(self.count(devices, "BLUETOOTH"), 0)
+        self.assertEqual(self.count(devices, "NETWORK"), 1)
+        self.assertEqual(self.count(devices, "WIRELESS"), 1)
+        self.assertEqual(len(devices), 71)
+        # First card is an Intel Xeon E3-1200 v2/3rd Gen Core processor
+        # Graphics Controller Second one is NVidia  GF119 [GeForce GT 620 OEM]
+        expected_devices = [
+            (None, "VIDEO", "pci", 0x8086, 0x0152),
+            (None, "VIDEO", "pci", 0x10de, 0x1049),
+            ("RTL8111/8168/8411 PCI Express Gigabit Ethernet Controller",
+             "NETWORK", "pci", 0x10EC, 0x8168),
+            ]
+        self.verify_devices(devices, expected_devices)
+
     def verify_devices(self, devices, expected_device_list):
         """
         Verify we have exactly one of each device given in the list,
